--
-- xmonad example config file.
--
-- A template showing all available configuration hooks,
-- and how to override the defaults in your own xmonad.hs conf file.
--
-- Normally, you'd only override those defaults you care about.
--

import XMonad
import Data.Monoid
import System.Exit

import XMonad.Hooks.StatusBar
import XMonad.Hooks.StatusBar.PP

import qualified XMonad.StackSet as W
import qualified Data.Map        as M

import XMonad.Layout.Spacing
import XMonad.Layout.Tabbed

import XMonad.Actions.KeyRemap

import XMonad.Prompt
import XMonad.Prompt.Layout
import Graphics.X11.ExtraTypes.XF86

import Control.Monad (when)

-- The preferred terminal program, which is used in a binding below and by
-- certain contrib modules.
--
myTerminal = "kitty"

-- Whether focus follows the mouse pointer.
myFocusFollowsMouse :: Bool
myFocusFollowsMouse = False

-- Whether clicking on a window to focus also passes the click to the window
myClickJustFocuses :: Bool
myClickJustFocuses = False

-- Width of the window border in pixels.
--
myBorderWidth = 2

-- modMask lets you specify which modkey you want to use. The default
-- is mod1Mask ("left alt").  You may also consider using mod3Mask
-- ("right alt"), which does not conflict with emacs keybindings. The
-- "windows key" is usually mod4Mask.
--
myModMask = mod4Mask

-- The default number of workspaces (virtual screens) and their names.
-- By default we use numeric strings, but any string may be used as a
-- workspace name. The number of workspaces is determined by the length
-- of this list.
--
-- A tagging example:
--
-- > workspaces = ["web", "irc", "code" ] ++ map show [4..9]
--
myWorkspaces = ["comms","browser","3","4","5","6","7","8","9"]

-- Border colors for unfocused and focused windows, respectively.
--
myNormalBorderColor  = "#3c3836"
myFocusedBorderColor = "#504945"

myRemaps = KeymapTable [ ((0, xK_a), (0, xK_b)) ]

------------------------------------------------------------------------
-- Key bindings. Add, modify or remove key bindings here.
--
myKeys conf@(XConfig {XMonad.modMask = modm}) = M.fromList $
  [ ((modm, xK_Return), spawn $ XMonad.terminal conf)
  , ((modm .|. shiftMask, xK_q), kill)
  -- Rotate through the available layout algorithms
  , ((modm,               xK_space ), sendMessage NextLayout)
  -- rofiing
  , ((modm, xK_d), spawn "rofi -show drun")
  , ((modm .|. shiftMask, xK_e), spawn "rofi -show \"desktopctl\" -modes \"desktopctl:$(which desktopctl)\"")
  , ((modm, xK_m), spawn "menu-qalc")
  , ((modm, xK_i), spawn "rofimoji -f alchemical_symbols anatolian_hieroglyphs emojis braille_patterns box_drawing chess_symbols emoticons geometric_shapes gothic greek_extended math mathematical_alphanumeric_symbols mathematical_operators miscellaneous_symbols miscellaneous_mathematical_symbols-a miscellaneous_mathematical_symbols-b miscellaneous_symbols_and_arrows miscellaneous_symbols_and_pictographs miscellaneous_technical modi modifier_tone_letters musical_symbols nerd_font number_forms shorthand_format_controls specials variation_selectors vertical_forms -a copy")
  -- screenshotting
  -- TODO: Fix/rewrite window-screenshot.sh
  , ((modm, xK_w), spawn "window-screenshot.sh")
  , ((modm, xK_s), spawn "flameshot gui -c -p $HOME/Pictures/screenshots")
  , ((modm, xK_a), spawn "flameshot screen -c -p $HOME/Pictures/screenshots")
  , ((modm, xK_t), spawn "ocr-screenshot.sh")

  [ ((modm, xK_Return), spawn $ XMonad.terminal conf)
  , ((modm .|. shiftMask, xK_q), kill)
   -- Rotate through the available layout algorithms
  , ((modm,               xK_space ), sendMessage NextLayout)

	-- rofiing
  , ((modm, xK_d), spawn "rofi -show drun")
	, ((modm .|. shiftMask, xK_e), spawn "rofi -show desktopctl -modes desktopctl:desktopctl -show-icons")
	, ((modm, xK_m), spawn "menu-qalc -- -theme gruvbox-dark")
	, ((modm, xK_i), spawn "rofimoji --selector-args '-theme gruvbox-dark' -f alchemical_symbols anatolian_hieroglyphs emojis braille_patterns box_drawing chess_symbols emoticons geometric_shapes gothic greek_extended math mathematical_alphanumeric_symbols mathematical_operators miscellaneous_symbols miscellaneous_mathematical_symbols-a miscellaneous_mathematical_symbols-b miscellaneous_symbols_and_arrows miscellaneous_symbols_and_pictographs miscellaneous_technical modi modifier_tone_letters musical_symbols nerd_font number_forms shorthand_format_controls specials variation_selectors vertical_forms -a copy")

  , ((modm, xK_o), spawn "rofi -show searchwolf -modes searchwolf:searchwolf -theme gruvbox-dark")

	-- screenshotting
	-- TODO: Fix/rewrite window-screenshot.sh
	, ((modm, xK_w), spawn "window-screenshot.sh")
	, ((modm, xK_s), spawn "flameshot gui -c -p $HOME/Pictures/screenshots")
	, ((modm, xK_a), spawn "flameshot screen -c -p $HOME/Pictures/screenshots")
	, ((modm, xK_t), spawn "ocr-screenshot.sh")

  , ((modm .|. shiftMask, xK_t), spawn "DEVICE=\"MELF0410:00 1FD2:7007\"; if [ $(xinput list-props \"$DEVICE\" | awk '/^\\tDevice Enabled \\([0-9]+\\):\\t[01]/ {print $NF}') = \"1\" ]; then xinput disable \"$DEVICE\"; else xinput enable \"$DEVICE\"; fi")

  --  Reset the layouts on the current workspace to default
  -- , ((modm .|. shiftMask, xK_space ), setLayout $ XMonad.layoutHook conf)
  -- , ((modm .|. shiftMask, xK_space ), setLayout $ XMonad.layoutHook conf)
  -- Push window back into tiling
  , ((modm .|. shiftMask, xK_space ), withFocused $ windows . W.sink)


  -- Resize viewed windows to the correct size
  , ((modm,               xK_n     ), refresh)

  -- Move focus to the next window
  , ((modm,               xK_Tab   ), layoutPrompt def)

  -- Move focus
  , ((modm,               xK_j     ), windows W.focusDown)
  , ((modm,               xK_k     ), windows W.focusUp  )

  -- Swap the focused window with the next window
  , ((modm .|. shiftMask, xK_j     ), windows W.swapDown  )
  , ((modm .|. shiftMask, xK_k     ), windows W.swapUp    )

  -- Shrink the master area
  , ((modm,               xK_h     ), sendMessage Shrink)
  , ((modm,               xK_l     ), sendMessage Expand)

  -- Increment the number of windows in the master area
  , ((modm              , xK_comma ), sendMessage (IncMasterN 1))
  , ((modm              , xK_period), sendMessage (IncMasterN (-1)))

  -- Brightness n stuff 
  -- , ((0, xF86XK_AudioRaiseVolume),  spawn "amixer -D pulse sset Master 10%+")
  -- , ((0, xF86XK_AudioLowerVolume),  spawn "amixer -D pulse sset Master 10%-")
  -- , ((0, xF86XK_AudioMute),         spawn "amixer -D pulse sset Master toggle")
  -- , ((0, xF86XK_MonBrightnessUp),   spawn "brightnessctl set +10%")
  -- , ((0, xF86XK_MonBrightnessDown), spawn "brightnessctl set 10%-")
  -- , ((0, xF86XK_KbdBrightnessUp),   spawn "brightnessctl -d \"kbd_backlight\" set +10%")
  -- , ((0, xF86XK_KbdBrightnessDown), spawn "brightnessctl -d \"kbd_backlight\" set 10%-")

  -- Toggle the status bar gap
  -- Use this binding with avoidStruts from Hooks.ManageDocks.
  -- See also the statusBar function from Hooks.DynamicLog.
  --
  -- , ((modm              , xK_b     ), sendMessage ToggleStruts)

  --, ((modm .|. shiftMask, xK_e     ), io (exitWith ExitSuccess))

  -- Restart xmonad
  --, ((modm              , xK_q     ), spawn "xmonad --recompile; xmonad --restart")

  -- Run xmessage with a summary of the default keybindings (useful for beginners)
  ]
  ++

  --
  -- mod-[1..9], Switch to workspace N
  -- mod-shift-[1..9], Move client to workspace N
  --
  [((m .|. modm, k), windows $ f i)
      | (i, k) <- zip (XMonad.workspaces conf) [xK_1 .. xK_9]
      , (f, m) <- [(W.greedyView, 0), (W.shift, shiftMask)]]
  -- ++

  --
  -- mod-{w,e,r}, Switch to physical/Xinerama screens 1, 2, or 3
  -- mod-shift-{w,e,r}, Move client to screen 1, 2, or 3
  --
  -- [((m .|. modm, key), screenWorkspace sc >>= flip whenJust (windows . f))
  --     | (key, sc) <- zip [xK_w, xK_e, xK_r] [0..]
  --     , (f, m) <- [(W.view, 0), (W.shift, shiftMask)]]


------------------------------------------------------------------------
-- Mouse bindings: default actions bound to mouse events
--
myMouseBindings (XConfig {XMonad.modMask = modm}) = M.fromList $
  -- mod-button1, Set the window to floating mode and move by dragging
  [ ((modm, button1), (\w -> focus w >> mouseMoveWindow w
                                     >> windows W.shiftMaster))

  -- mod-button2, Raise the window to the top of the stack
  , ((modm, button2), (\w -> focus w >> windows W.shiftMaster))

  -- mod-button3, Set the window to floating mode and resize by dragging
  , ((modm, button3), (\w -> focus w >> mouseResizeWindow w
                                     >> windows W.shiftMaster))

  -- you may also bind events to the mouse scroll wheel (button4 and button5)
  ]

------------------------------------------------------------------------
-- Layouts:

-- You can specify and transform your layouts by modifying these values.
-- If you change layout bindings be sure to use 'mod-shift-space' after
-- restarting (with 'mod-q') to reset your layout state to the new
-- defaults, as xmonad preserves your old layout settings by default.
--
-- The available layouts.  Note that each layout is separated by |||,
-- which denotes layout choice.

tabCfg = def 
  { activeColor = "#282828"
  , inactiveColor = "#1d2021"
  , urgentColor = "#9d0006"
  , activeBorderColor = "#504945"
  , inactiveBorderColor = "#3c3836"
  , urgentBorderColor = "#cc241d"
  , activeTextColor = "#ebdbb2"
  , inactiveTextColor = "#bdae93"
  , urgentTextColor = "#ebdbb2"
  }
myLayout = tiled ||| tabbed shrinkText tabCfg ||| Mirror tiled 
  where
     -- default tiling algorithm partitions the screen into two panes
     tiled   = Tall nmaster delta ratio

     -- The default number of windows in the master pane
     nmaster = 1

     -- Default proportion of screen occupied by master pane
     ratio   = 1/2

     -- Percent of screen to increment by when resizing panes
     delta   = 3/100

------------------------------------------------------------------------
-- Window rules:

-- Execute arbitrary actions and WindowSet manipulations when managing
-- a new window. You can use this to, for example, always float a
-- particular program, or have a client always appear on a particular
-- workspace.
--
-- To find the property name associated with a program, use
-- > xprop | grep WM_CLASS
-- and click on the client you're interested in.
--
-- To match on the WM_NAME, you can use 'title' in the same way that
-- 'className' and 'resource' are used below.
--
myManageHook = composeAll
  [ className =? "MPlayer"        --> doFloat
  , className =? "Gimp"           --> doFloat
  , resource  =? "desktop_window" --> doIgnore
  , resource  =? "kdesktop"       --> doIgnore ]

------------------------------------------------------------------------
-- Event handling

-- * EwmhDesktops users should change this to ewmhDesktopsEventHook
--
-- Defines a custom handler function for X Events. The function should
-- return (All True) if the default handler is to be run afterwards. To
-- combine event hooks use mappend or mconcat from Data.Monoid.
--
myEventHook = mempty

focusChangeHook (FocusChangeEvent {ev_event_type=t, ev_window=window}) = do 
  when (t == focusIn) $ spawn "notify-send 'focusIn'"
  when (t == focusOut) $ spawn "notify-send 'focusOut'"
  return $ All True

------------------------------------------------------------------------
-- Status bars and logging

-- Perform an arbitrary action on each internal state change or X event.
-- See the 'XMonad.Hooks.DynamicLog' extension for examples.
--
myLogHook = return ()

------------------------------------------------------------------------
-- Startup hook

-- Perform an arbitrary action each time xmonad starts or is restarted
-- with mod-q.  Used by, e.g., XMonad.Layout.PerWorkspace to initialize
-- per-workspace layout choices.
--
-- By default, do nothing.
myStartupHook = do
  spawn "pkill oneko; oneko -tofocus -tora -position +-1+5 -fg palevioletred3 -bg pink -name 'Rose the desktop kitty'"
  --spawn "pkill polybar; polybar"
  spawn "pkill volumeicon; volumeicon"
  spawn "pkill nm-applet; nm-applet"
  spawn "mullvad-vpn --background"
  setDefaultKeyRemap myRemaps [myRemaps]



------------------------------------------------------------------------
-- Now run xmonad with all the defaults we set up.

-- Run xmonad with the settings you specify. No need to modify this.
--
mySB = statusBarProp "polybar" (pure xmobarPP) 
main = xmonad $ withEasySB mySB defToggleStrutsKey defaults

-- A structure containing your configuration settings, overriding
-- fields in the default config. Any you don't override, will
-- use the defaults defined in xmonad/XMonad/Config.hs
--
-- No need to modify this.
--
defaults = def 
  -- simple stuff
	{ terminal           = myTerminal
  , focusFollowsMouse  = myFocusFollowsMouse
  , clickJustFocuses   = myClickJustFocuses
  , borderWidth        = myBorderWidth
  , modMask            = myModMask
  , workspaces         = myWorkspaces
  , normalBorderColor  = myNormalBorderColor
  , focusedBorderColor = myFocusedBorderColor
  -- , clientMask = focusChangeMask .|. XMonad.clientMask 
  -- key bindings
  , keys               = myKeys
	, mouseBindings      = myMouseBindings
  -- hooks, layouts
	, layoutHook         = spacingRaw True (Border 0 0 0 0) True (Border 7 7 7 7) True $ myLayout
	, manageHook         = myManageHook
	, handleEventHook    = myEventHook
	, logHook            = myLogHook
	, startupHook        = myStartupHook
  }
